#include "Game.h"



Game::Game(SDL_Window* window, SDL_Renderer* renderer, int windowWidth, int windowHeight) :
    gameModeCurrent(GameMode::loading), timerLoading(3.0f), timerFinished(1.5f) {

    //Run the game.
    if (window != nullptr && renderer != nullptr) {
        //Load the some important textures.
        textureFunOSLoadingLogo = TextureLoader::loadTexture(renderer, "FunOS Loading Logo.bmp");
        textureBottomBar = TextureLoader::loadTexture(renderer, "Bottom Bar.bmp");
        textureBattleTime = TextureLoader::loadTexture(renderer, "Battle Time.bmp");
        textureCommandPrompt = TextureLoader::loadTexture(renderer, "Command Prompt.bmp");
        textureErrorMessage = TextureLoader::loadTexture(renderer, "Error Message.bmp");
        
        //Load the units.
        loadUnits(renderer, windowWidth, windowHeight);

        //Set up the timers that transition from game mode to game mode.
        timerLoading.resetToMax();
        timerFinished.resetToMax();


        //Store the current times for the clock.
        auto time1 = std::chrono::system_clock::now();
        auto time2 = std::chrono::system_clock::now();

        //The amount of time for each frame (60 fps).
        const float dT = 1.0f / 60.0f;


        //Start the game loop and run until it's time to stop.
        bool running = true;
        while (running) {
            //Determine how much time has elapsed since the last frame.
            time2 = std::chrono::system_clock::now();
            std::chrono::duration<float> timeDelta = time2 - time1;
            float timeDeltaFloat = timeDelta.count();

            //If enough time has passed then do everything required to generate the next frame.
            if (timeDeltaFloat >= dT) {
                //Store the new time for the next frame.
                time1 = time2;

                processEvents(running);
                update(dT);
                draw(renderer, windowWidth, windowHeight);
            }
        }
    }
}


Game::~Game() {
    //Clean up.
    TextureLoader::deallocateTextures();
    SoundLoader::deallocateSounds();
}


void Game::loadUnits(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    //Load units randomly on a grid.
    const int spacing = (int)round(64 * 1.5f);
    const int countXMax = windowWidth / spacing;
    const int countYMax = windowHeight / spacing - 1;

    for (int countX = 0; countX < countXMax; countX++) {
        for (int countY = 0; countY < countYMax; countY++) {
            //A random chance to add a unit at the specified position.
            if (rand() % 3 == 0) {
                //Setup the unit's position.
                Vector2D pos((float)(countX * spacing + spacing / 2),
                    (float)(countY * spacing + spacing / 2));

                //Create and add the unit.  There's a random chance to be a folder or a file.
                listUnits.push_back(std::make_shared<Unit>(renderer, pos,
                    ((rand() % 2 == 0) ? Unit::Team::folder : Unit::Team::file)));
            }
        }
    }
}



void Game::processEvents(bool& running) {
    //Process events.
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
        switch (event.type) {
        case SDL_QUIT:
            running = false;
            break;

        case SDL_KEYDOWN:
            switch (event.key.keysym.scancode) {
                //Quit the game.
            case SDL_SCANCODE_ESCAPE:
                running = false;
                break;
            }

            //If final finished screen is visible and any key is pressed then quit the game.
            if (gameModeCurrent == GameMode::finished)
                running = false;

            //If the command prompt is visible and any key is pressed then switch the game mode.
            if (gameModeCurrent == GameMode::commandPrompt)
                gameModeCurrent = GameMode::playing;
        }
    }
}



void Game::update(float dT) {
    switch (gameModeCurrent) {
    case GameMode::loading:
        //Update the timer that counts from when the game starts until the command prompt is displayed.
        timerLoading.countDown(dT);
        if (timerLoading.timeSIsZero())
            gameModeCurrent = GameMode::commandPrompt;
        break;


    case GameMode::playing:
        //Update the units and check if the game is over.
        updateUnits(dT);

        if (checkIfGameFinished())
            gameModeCurrent = GameMode::gameOver;
        break;

        
    case GameMode::gameOver:
        //Update the timer that counts from when the game is over until the finished overlay is drawn.
        timerFinished.countDown(dT);
        if (timerFinished.timeSIsZero())
            gameModeCurrent = GameMode::finished;
        break;
    }
}


void Game::updateUnits(float dT) {
    for (int count = 0; count < listUnits.size(); count++) {
        auto& unitSelected = listUnits[count];
        if (unitSelected != nullptr) {
            unitSelected->update(dT, listUnits);
            if (unitSelected->isAlive() == false) {
                listUnits.erase(listUnits.begin() + count);
                count--;
            }
        }
    }
}



void Game::draw(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    //Draw.
    //Set the draw color to a black.
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    //Clear the screen.
    SDL_RenderClear(renderer);


    //Draw everything here.
    switch (gameModeCurrent) {
    case GameMode::loading:
        drawLoading(renderer, windowWidth, windowHeight);
        break;
    case GameMode::commandPrompt:
        drawCommandPrompt(renderer, windowWidth, windowHeight);
        break;
    case GameMode::playing:
    case GameMode::gameOver:
        drawPlaying(renderer, windowWidth, windowHeight);
        break;
    case GameMode::finished:
        drawFinished(renderer, windowWidth, windowHeight);
        break;
    }

    //Send the image to the window.
    SDL_RenderPresent(renderer);
}



void Game::drawLoading(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    //Set the draw color to a black and clear the screen.
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);

    //Draw the FunOS loading logo.
    if (textureFunOSLoadingLogo != nullptr) {
        int w = 0, h = 0;
        SDL_QueryTexture(textureFunOSLoadingLogo, NULL, NULL, &w, &h);
        SDL_Rect rect = { (windowWidth - w) / 2, (windowHeight - h) / 2, w, h };
        SDL_RenderCopy(renderer, textureFunOSLoadingLogo, NULL, &rect);
    }
}



void Game::drawCommandPrompt(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    drawPlaying(renderer, windowWidth, windowHeight);

    //Draw the command prompt.
    if (textureCommandPrompt != nullptr) {
        int w = 0, h = 0;
        SDL_QueryTexture(textureCommandPrompt, NULL, NULL, &w, &h);
        SDL_Rect rect = { (windowWidth - w) / 2, (windowHeight - h) / 2, w, h };
        SDL_RenderCopy(renderer, textureCommandPrompt, NULL, &rect);
    }
}



void Game::drawPlaying(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    //Set the draw color to a light blue and clear the screen.
    SDL_SetRenderDrawColor(renderer, 0, 128, 223, 255);
    SDL_RenderClear(renderer);

    //Draw the bottom bar.
    if (textureBottomBar != nullptr) {
        int w = 0, h = 0;
        SDL_QueryTexture(textureBottomBar, NULL, NULL, &w, &h);
        SDL_Rect rect = { 0, windowHeight - h, windowWidth, h };
        SDL_RenderCopy(renderer, textureBottomBar, NULL, &rect);
    }
    //Draw the battle time overlay over the bottom bar.
    if (textureBattleTime != nullptr) {
        int w = 0, h = 0;
        SDL_QueryTexture(textureBattleTime, NULL, NULL, &w, &h);
        SDL_Rect rect = { windowWidth - w - 18, windowHeight - h - 12, w, h };
        SDL_RenderCopy(renderer, textureBattleTime, NULL, &rect);
    }

    //Draw the units.
    for (auto& unitSelected : listUnits)
        if (unitSelected != nullptr)
            unitSelected->draw(renderer);
}



void Game::drawFinished(SDL_Renderer* renderer, int windowWidth, int windowHeight) {
    //Set the draw color to a dark blue and clear the screen.
    SDL_SetRenderDrawColor(renderer, 0, 71, 224, 255);
    SDL_RenderClear(renderer);

    //Draw error message.
    if (textureErrorMessage != nullptr) {
        int w = 0, h = 0;
        SDL_QueryTexture(textureErrorMessage, NULL, NULL, &w, &h);
        SDL_Rect rect = { (windowWidth - w) / 2, (windowHeight - h) / 2, w, h };
        SDL_RenderCopy(renderer, textureErrorMessage, NULL, &rect);
    }
}



bool Game::checkIfGameFinished() {
    //Check if there is at least one unit on each team or not.
    return (teamStillHasAtLeastOneUnit(Unit::Team::folder) == false ||
        teamStillHasAtLeastOneUnit(Unit::Team::file) == false);
}



bool Game::teamStillHasAtLeastOneUnit(Unit::Team team) {
    //Check the input team to see if it has at least one unit.
    for (auto& unitSelected : listUnits)
        if (unitSelected != nullptr && unitSelected->getTeam() == team)
            return true;

    return false;
}