#include "Unit.h"


const float Unit::speed = 100.0f;




Unit::Unit(SDL_Renderer* renderer, Vector2D setPos, Team setTeam) :
	pos(setPos), team(setTeam), timerJustHurt(0.25f), timerWeapon(1.0f),
	texture(TextureLoader::loadTexture(renderer, (setTeam == Team::folder ? "Folder.bmp" : "File.bmp"))),
	mix_ChunkAttack(SoundLoader::loadSound("Paper Attack.ogg")),
	mix_ChunkDie(SoundLoader::loadSound("Paper Crumple.ogg")) {

}



void Unit::update(float dT, std::vector<std::shared_ptr<Unit>>& listUnits) {
	//Update timers.
	timerJustHurt.countDown(dT);
	timerWeapon.countDown(dT);

	//If a target has been found but is no longer alive then reset it.
	if (auto unitTargetSP = unitTarget.lock())
		if (unitTargetSP->isAlive() == false)
			unitTarget.reset();

	//A random chance to find a target if one hasn't already been found.
	if (unitTarget.expired() && (rand() % 120 == 0))
		unitTarget = findEnemyUnit(listUnits);


	//Check if a target has been found.
	if (auto unitTargetSP = unitTarget.lock()) {
		//Check how far away this unit is from it's target.
		Vector2D directionTarget = (unitTargetSP->pos - pos);
		const float distanceToTarget = directionTarget.magnitude();
		const float distanceDesired = 64 * 1.25f;

		//Check if this unit is close enough to it's target.
		if (distanceToTarget > distanceDesired) {
			//It's too far.
			//The distance to move this frame.
			float distanceMove = speed * dT;
			//Move this unit.
			pos += directionTarget.normalize() * distanceMove;
		}
		else {
			//It's close enough.
			//Attack the target if the weapon timer is ready.
			if (timerWeapon.timeSIsZero()) {
				//Play the attack sound.
				if (mix_ChunkAttack != nullptr)
					Mix_PlayChannel(-1, mix_ChunkAttack, 0);

				unitTargetSP->removeHealth(1);
				//Check if the target is still alive and if not then reset it.
				if (unitTargetSP->isAlive() == false)
					unitTarget.reset();

				timerWeapon.resetToMax();
			}
		}
	}
}



void Unit::draw(SDL_Renderer* renderer) {
	if (renderer != nullptr && texture != nullptr) {
		//Set the color to red if needed.
		if (timerJustHurt.timeSIsZero() == false)
			SDL_SetTextureColorMod(texture, 255, 0, 0);
		else
			SDL_SetTextureColorMod(texture, 255, 255, 255);

		//Draw a rectangle at the unit's position.
		SDL_Rect rect = {
			(int)(pos.x - 64 / 2),
			(int)(pos.y - 64 / 2),
			64,
			64 };
		SDL_RenderCopy(renderer, texture, NULL, &rect);
	}
}



bool Unit::isAlive() {
	return (healthCurrent > 0);
}



std::weak_ptr<Unit> Unit::findEnemyUnit(std::vector<std::shared_ptr<Unit>>& listUnits) {
	std::weak_ptr<Unit> closestUnit;
	float distanceClosest = 0.0f;

	//Loop through the entire list of units.
	for (auto& unitSelected : listUnits) {
		//Ensure that the selected unit exists and is on the other team.
		if (unitSelected != nullptr && unitSelected->team != team) {
			float distanceCurrent = (pos - unitSelected->pos).magnitude();
			//Check if no closest unit has been found or the selected unit is closer than the 
			//previous closest unit.
			if (closestUnit.expired() || distanceCurrent < distanceClosest) {
				//Then set the closest unit to the selected unit.
				closestUnit = unitSelected;
				distanceClosest = distanceCurrent;
			}
		}
	}

	return closestUnit;
}



void Unit::removeHealth(int damage) {
	if (damage > 0) {
		int healthLast = healthCurrent;

		healthCurrent -= damage;
		if (healthCurrent < 0)
			healthCurrent = 0;

		timerJustHurt.resetToMax();

		if (mix_ChunkDie != nullptr && healthCurrent == 0 && healthLast > 0)
			Mix_PlayChannel(-1, mix_ChunkDie, 0);
	}
}



Unit::Team Unit::getTeam() {
	return team;
}